@@ -12,12 +12,13 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import mock
+
+from reno import config
 from reno import formatter
 from reno import loader
 from reno.tests import base
 
-import mock
-
 
 class TestFormatter(base.TestCase):
 
@@ -58,13 +59,11 @@ class TestFormatter(base.TestCase):
                 'file-contents': self.note_bodies
             }
 
+        self.c = config.Config('reporoot')
+
         with mock.patch('reno.loader.Loader._load_data', _load):
             self.ldr = loader.Loader(
-                reporoot='reporoot',
-                notesdir='notesdir',
-                branch=None,
-                collapse_pre_releases=None,
-                earliest_version=None,
+                self.c,
                 ignore_cache=False,
             )
 
